
/**********************************************************************
;                                                                     *
;    Filename:	    picFruit.h	(frutti Skynet su 485/wireless)       *
;    Date:          06.2017 (28/2/05)                                 *
;    File Version:  4.00                                              *
;                                                                     *
;    Author:        Dario/Cinzia Greggio                              *
 * Company:        Oetztal Automation (Cyberdyne (ADPM Synthesis sas)) *
;                                                                     *
;**********************************************************************/

#if defined(__PIC24FV32KA301__) || defined(__PIC24FV32KA302__)
#include <p24fxxxx.h>
#else
#include <xc.h>
#endif

#include <string.h>
#include "generictypedefs.h"
#include "wireless_2400.h"								// per MAX_HOPS, cmq

#define TRANSMITTER_ADDRESS 201
#define RECEIVER_ADDRESS 202

//#define MRF24J40MB 1

#if defined(__PIC24FV32KA302__) || defined(__PIC24FJ256GA702__) || defined(__PIC24EP512GP202__) 
#define USE_HARDWARE_PWM 1
#endif

#define INVALID_READOUT 9999		//meglio di ffff !!


#if defined(__PIC24FV32KA301__) || defined(__PIC24FV32KA302__) || defined(__PIC24FJ256GA702__)
#define CLOCK_FREQ 32000000ULL
#else
#define CLOCK_FREQ 140000000ULL
#endif
#define GetSystemClock()		(CLOCK_FREQ)      // Hz
#define GetInstructionClock()	(GetSystemClock()/2)
#define GetPeripheralClock()	GetInstructionClock()
#define FCY (GetSystemClock()/2)		// per LibPic30.h e delay


#define MY_ADDRESS 18	// per MiWi e MAC...


struct SAVED_PARAMETERS {
	BYTE MyHouse;    // indirizzo casa (0..15) : OCCHIO che se per caso viene messo a >15 (p.es EEprom alla programmazione) non lo becchi +!!
	BYTE MyAddress;  // indirizzo nodo (0..255)
	BYTE MyPty;      // priorita' nell'accesso alla rete (CSMA) 1..3
	/*volatile*/ BYTE MyCfg[4];  // bit 0=attiva funz. locale puls.1 
											// bit 1=attiva funz. locale puls.2&3
											// bit 2= notify press (puls.1/2/3/4) 
											// bit 3= notify press.ext. (click doppio/prolungato (con repeat)) (puls.1/2/3/4)
											// bit 4= notify rel (puls.1/2/3/4): 
											// bit 5= beep di conferma press. tasti (FINIRE!)
											// bit 6= 1: SPI/I2C, 0: 232 (attiva ricezione con IRQ-portB)
											// bit 7= attiva SPI/I2C/232; IGNORATO da questa versione in poi (ossia sempre usabile

											// bit 1-0 funz. iDimmer
											// bit 2=master/slave dimmer
											// bit 3=soft ON/OFF
											// bit 4=lampegg. usc.2
											// bit 5=lampeggiante usc.3
											// bit 6=autospegni su sovraccarico
											// bit 7=sens.IR sull'input 1 (sens. chiuso=luce spenta, sens. aperto=luce accesa; usa anche il crepuscolare se attivato; usa il funz. locale, se attivo, per uscita 1 legata a IR)

											// (per wireless...) bit 0 = ripeti pacchetti non miei
											// (per wireless...) bit 1 = usa FHSS/spread spectrum
											// (per wireless...) bit 7 = modalit "dormant" va in sleep e si accende ogni tanto
//										 bit4 MUTUAL_EXCLUDE usc0 & 1... per tapparella 

											// per avere dinamicamente dimmer/pwm/io semplice
											// 00=i/o
											// 01=dimmer AC
											// 10=dimmer motori
											// 11=PWM
											// 	(replicato per max 4 uscite)

	BYTE Value[4];		// 0..255, 2017
	BYTE GenericValue;
	BYTE saveValue[4];
	BYTE ADRange;			// (ofs 16) 4 bit alti=limite High, 4 bit bassi=limite Low; 0x0=nulla, 0xff=sempre
	BYTE NotifDevice;		// dispositivo I2C (gi shiftato, senza bit 0) da cui leggere automaticamente (v. casi particolari)
	BYTE NotifTimer;
	BYTE OVLThrs;			// bit7 abilita notify, 0..127 e' la soglia di sovraccarico (v. anche bit in MyCfg per autospegnimento)
	volatile BYTE powerFail;			// bit 0 attivato ogni tot secondi e pulito dall'IRQ 50Hz, bit 7 indica notificare se manca 220V
	BYTE delayUART;
	BYTE Gruppo;				// 2 gruppi, 0..15
	BYTE InvertiInput;		// 
	BYTE ID[4];						// array di 4 char come identificativo
#if defined(WIRELESS_MIWI) || defined(WIRELESS_RF24) || defined(WIRELESS_ER900) || defined(WIRELESS_MC13192) || defined(WIRELESS_CC2420)
	BYTE routingTable[MAX_HOPS];		// questa  da/verso il coordinatore... potrebbero essercene altre inter-frutti??
#endif
	};

extern struct SAVED_PARAMETERS configParms;

#define PARMS_FIELD(field) configParms.field

#define BUF_SIZE   32 				// include tutta una Packetxxx (da 13) + ttl e hops (su wireless), e ci sta anche il jumbo packet 47
union PACKETS {
	BYTE Buffer[BUF_SIZE];		// i BYTE ricevuti da seriale
	struct {
		struct {								// i BYTE del buffer... v. BUF_SIZE
			unsigned char house:4;
			unsigned char priority:2;
			unsigned char version:2;
			} Header;
		BYTE DstAddr;
		BYTE SrcAddr;
		struct {
			unsigned char frameCnt:7;
			unsigned char ackWanted:1;
			} fcf;
		union {
			struct {
				// right to left, lsb to msb, Microsoft specific...
				unsigned char subCode:5;
				unsigned char subGroup:3;
				};
			BYTE Cmd;
			};
#if defined(WIRELESS_MIWI) || defined(WIRELESS_RF24) || defined(WIRELESS_ER900) || defined(WIRELESS_MC13192) || defined(WIRELESS_CC2420)
		BYTE Data[BUF_SIZE-5-1-1-MAX_HOPS];
		BYTE ChkSum;			// non  sempre qui.. ma va bene!
		BYTE ttl;					// forse dovrebbe stare DENTRO il chksum (=casino/rifare tutti i PICFruit) per qua il wireless ci salva con il suo checksum...
		BYTE hops[MAX_HOPS];
#else
		BYTE Data[BUF_SIZE-5-1-1];
#endif
		};
	};

signed char DoI2CMO(BYTE);
BYTE DoI2CMI(BYTE);
void DoI2CStart(void);
void DoI2CStop(void);
void DoSHTStart(void);
void DoSHTStop(void);
void DoSPIMO(BYTE);
BYTE DoSPIMI(void);
void DoSPIStart(void);
void DoSPIStop(void);

void Do232O(BYTE);
BYTE Do232I(void);
void DoIROut(BYTE,BYTE,BYTE,BYTE *);
void DoIROut_delay_1(WORD);
void DoIROut_delay_0(WORD);
BYTE DoIRIn0(void);
BYTE DoIRIn1(void);

void DallasOut(BYTE);
BYTE DallasIn(void);

BYTE EseguiComando(BYTE);
BYTE RiceviFrame(void);

// #define SetFram0(n,d) { SetFrame(n,d,0,0); }								// non + usata
BYTE SetFrame(BYTE,BYTE,BYTE,BYTE);

BYTE MandaHdr(void);
void EndFrame(void);
unsigned short int GetByte(void);

WORD getBRGFromBaudRate(DWORD );

#define PutByte0 PutByte(0)               // manda uno 0
void PutByte(BYTE);

BYTE subSpedisciFrame(union PACKETS *);
BYTE SpedisciFrame(union PACKETS *);
BOOL GetParita(BYTE);

//#define SetFrameNotifPuls1(n) SetFrameNotifPuls(0,n)
BYTE SetFrameNotifPuls(BYTE,BYTE);
BYTE SetFrameNotifIR(BYTE,BYTE);
BYTE getPulsStatus(void);
void clrACK(void);
void initLVD(void);
BYTE initPWM(void);
BYTE initADC(void);
void initPullups(void);
signed char initZeroCrossing(void);
signed char initRS232(void);
BYTE initComm(BYTE channel);

//#ifdef I_AM_TRANSMITTER
BYTE scanKBD(void);
//#endif

void GestPulsanti(BYTE, BYTE);


//typedef void * SHORTPTR;			//
//#define GetChkSumO(len) GetChkSum(packetOut.Buffer,len)        // chksum buffer out (entra W=temp=n BYTE)
#define GetChkSumI(len) GetChkSum(packetIn.Buffer,len)					// chksum buffer in (entra W=temp=n BYTE)
BYTE GetChkSum(BYTE *,BYTE);

//void Delay_mS(BYTE );
#define Delay_uS(n) __delay_us(n)
void Delay_S_(BYTE);

#define EEscrivi0(p) EEscrivi(p,0)               // scrive 0...
#define EEscrivi(a,b) { *(((BYTE *)configParms)+a)=b; EEscrivi_(a,b); }  // scrive W in RAM e EPROM...
void EEscrivi_(BYTE,BYTE);
BYTE EEleggi(BYTE);
#define EEcopiaARAM(o) { *(((BYTE *)configParms)+o)=EEleggi(o); }
#define EEcopiaAEEPROM(n) { EEscrivi_(offsetof(struct SAVED_PARAMETERS,n),PARMS_FIELD(n)); }

#define CalcTriacSpento(q) CalcTriac(q,VAL_SPENTO)
#define CalcTriac(q,n) { configParms.Value[q]=n; CalcTri_(q); }
void CalcTri_(BYTE q);

#define Delay_1mS() __delay_ms(1)				
#define Delay_mS_free(n) __delay_ms(n)
#define Delay_1uS() __delay_us(1)

#define Delay08()	__delay_us(TIME_GRANULARITY)			// 1 bit-time 232

void waitClockHigh(void);
#define setDataOutput()	{ SPI_232_I_TRIS=0; }				// DATA  output
#define setDataInput() { SPI_232_I_TRIS=1; }				// DATA  input

#define Delay_I2C() __delay_us(4); //Delay_uS(DelaySPIVal)		// circa 100KHz, 8.8.17
// OCCHIO a chiamare __delay_us con una variabile, il codice diventa osceno!
	// 100KHz  OK per MicroChip 24LC16 (memorie) e TC74 (temp); Sensirion SHT11 va fino a oltre 1MHz; l'SMBus del chipset VIA andava a 16KHz; il clock-generator per PC 9248-95 va al max a 100KHz
#define Delay_SPI() { ClrWdt();Nop();Nop();Nop(); }//Delay_uS(DelaySPIVal)  // circa 1MHz, 23.4.17
// su PIC24E saranno circa 5MHz.. lascio cos

//#define Do232dl() Delay_uS(delayUART)
  // (su ST, a 11MHz 37 vale circa 2400baud=0.41mSec; v. Delay08; su PIC, in uS)
//#define Do232dl_(n) Delay_uS(n)
#define Do232dl() Do232dl_(configParms.delayUART)
#define Do232dl_(n) { TMR2=0; while(TMR2<(n/2)) ClrWdt();}
//2012: per evitare influenze da IRQ dimmer ecc. Timer2 (beep) viaggia a 1 ogni 2uS @32MHz

#define BUF_232_SIZE 32 		// 2^n !! 
BYTE CheckBuffer232(void);
void Flush232(BYTE);						// entra W=byte da copiare

signed int leggi_temp(BYTE);			// in decimi C
signed int leggi_tempMCP(BYTE);
WORD leggi_press(BYTE);
WORD leggi_AD(BYTE);
signed int leggi_tempSHT11(void);
WORD leggi_umidSHT11(signed char temp_comp /*C*/);
signed int leggi_tempSHT21(void);
WORD leggi_umidSHT21(signed char temp_comp /*C*/);
WORD leggi_tensione(BYTE);
WORD leggi_corrente(BYTE);

// ??nS/istruzione @ 40MHz (??)

//#define NO_PLUG_PLAY 1
//#define NO_ACK_WANTED 0      ; disabilita richiesta/invio ACK/NACK (utile per trasmissione non duplex)
//#define NO_EXT_ACK   0				; disabilita la risposta con ACK estesi da parte del frutto
#define NO_MS_ACK   1				// disabilita invio/ricez ACK tra frutti (master/slave)
//#define DEBUG_PULS  1					; reset ai default di alcuni valori se premuto il puls2 (no 1 che fa SENSORE INFRAROSSO e no 3 che fa SPI) all'accensione
#define MAX_RETRIES  3		// max= 15, min=1 (oppure 0 per disattivare)
#define RETRIES_TIME  4		// max= 15, 4=40mSec (v. loop), (era ~30 pre-2017)


#define PACKET_VERSION 0    // current version dei pacchetti
// PACKET_VERSION non dovra' MAI essere 11b (3) per evitare che il primo BYTE del pacchetto sia 0xff
// questo xche' ora usiamo 0xff come preambolo (nella 485, ma in pratica anche nelle convogliate)


//#define TEST_ENABLED 1


//#define USE_I2C 1				// 1=I2C; mah qua potremmo attivarli entrambi sempre?
//#define USE_SPI 1				// 1=SPI
#define IRQ_232 0				// 1=attiva IRQ per ricez. da 232 ; OPPURE 0!
// QUA (su picfruit 2011) SERVE PONTICELLO DA SDA a RB7
#define IRQ_I2C	0				// 1=attiva IRQ per req.ric. da I2C; OPPURE 0!
//#define PARITA_232	1					// attiva 8,O,1 RS232, altrimenti 8,N,1
//#define COMM_232_OR_485	1			// 1=485, usa un pin come TX_EN

//#define OUT1_DIMMER 1				// ecc non servono pi; abilita dimmer su OUT2

// LCD_GEST	EQU 1						// SEMPRE PRESENTI QUI! attiva comandi 33..35 e notifySer per LCD su I2C/SPI
#define LCD_ADDRESS 0x48			// PICBell!
// NO_BUZZER 1				// SEMPRE PRESENTE QUI! 
// #define SHT11 1									// SEMPRE modifica I2C (cmd.50->51) per SHT11 che non  esatt. I2C
#define USA_ADC 1			// fisso 2017, v. cmd 105!

//#define USE_IR_TX 1						// SEMPRE PRESENTE QUI! attiva IR out (cmd. 56)
//#define USE_IR_RX 1						// attiva IR in - sony
//#define OUT3_BLINKER 1			// l'uscita 3 (o la 6 su picfruit2011) lampeggia SEMPRE come il led di test 
#define IR_INVERTITO 1			// il sensore IR sul pin 1  NC (altrimenti NA): NAPION e' NA (attivo alto)
													// CAMBIA il comportamento e i msg del sensore IR!! (v.) e i pullup/down, 2017
//#define HIGH_SPEED_INPUT 1			; aumenta la velocit di lettura dgli input (per torni Marco, 2004)
#define SPI_USE_CS 1



#define TMR0BASE_DIMMER 1599
#define VAL_TRIACTABLE_SPENTO 93		// senza resistenza verso massa, 07.17
#define VAL_TRIACTABLE_ACCESO 6			// idem
//#define TMR1BASE  65536-541+40			// +qc per compensare!! (era per manchester)
// TMR1 viaggia a 1presc*(5MHz (200nS))
// io voglio un IRQ ogni 104uS (1200baud/Manchester con oversampling 4x) = 2165/4 = 541
#define TMR2BASE (25000)						//2012-2017: usato per time-oraesatta; 100mS
// v. anche IRQ, per PIC26K20 @64MHz...
#define TMR3BASE (640-1)						// AD sample in psyche, 60uS

#define TIME_GRANULARITY 9   // il tempo per 1 bit (8.7uS @ 115200baud)
													// max frame= 11bitx13BYTE(+3 pre/post)=1.55mS, + ACK=5(+3)BYTE=0.75mS

#define BEEP_STD_FREQ	230			// ca 4000Hz (ok per buzzer) @ ?? MHz!




// port A

#if defined(__PIC24FV32KA302__) || defined(__PIC24EP512GP202__) || defined(__PIC24FJ256GA702__)
#define Puls1Bit		0
#define m_Puls1Bit		PORTAbits.RA0
#define Puls2Bit		1
#define m_Puls2Bit		PORTAbits.RA1
#define Puls3Bit		2
#define m_Puls3Bit		PORTAbits.RA2
#define Puls4Bit		3
#define m_Puls4Bit		PORTAbits.RA3
#define AnaInBit    0 		//RA0
#define SinusInBit  4
#else
#define Puls1Bit		0			// questi sono CMQ in ordine 0..n (v. scanKBD() ! )
#define m_Puls1Bit		PORTAbits.RA1
#define Puls2Bit		1
#define m_Puls2Bit		PORTBbits.RB4
#define AnaInBit    1 		//RA0
#endif

// port B

#if defined(__PIC24FV32KA302__) || defined(__PIC24EP512GP202__) || defined(__PIC24FJ256GA702__)
//#if !defined(__PIC24FV32KA302__)
#define TriacO1Bit 7				// 	I/O 0
#define m_TriacO1Bit  LATBbits.LATB7
#define TriacO2Bit 11			// 	I/O 1
#define m_TriacO2Bit LATBbits.LATB11
#define TriacO3Bit 10			// 	I/O 2
#define m_TriacO3Bit LATBbits.LATB10
#define TriacO4Bit 2			// 	I/O 2
#define m_TriacO4Bit LATBbits.LATB2
#else
#define SinusInBit  0

#define TriacO1Bit 0			// 	I/O 0
#define m_TriacO1Bit LATBbits.LATB0
#define TriacO2Bit 1			// 	I/O 1
#define m_TriacO2Bit LATBbits.LATB1
#endif

#define Led1OBit  1				// I/O TTL 0
#define m_Led1OBit  LATBbits.LATB1		// 
//#define Led2OBit  1				// I/O TTL 1
//#define m_Led2OBit  LATBbits.LATB2		// 
//#define Led3OBit  2				// I/O TTL 2
//#define m_Led3OBit  LATBbits.LATB3



#define RXBit			2		// su UART
#define TXBit			7		// su UART
//CDBit     equ 3			qui no...

#if defined(__PIC24FV32KA302__) || defined(__PIC24EP512GP202__) || defined(__PIC24FJ256GA702__)
//#define TX_RXBit 15
#define m_TX_RXBit m_LedOBit			//LATBbits.LATB15			// siccome qua  CS, faccio cos
#define BuzzBit		5					// 
#define LedOBit		3			// attivo alto; 
#define m_LedOBit		LATBbits.LATB3
#else
#define TX_RXBit 15
#define m_TX_RXBit LATBbits.LATB15			// che non esiste fisicamente su chip 20 pin / pcb 2016 :)
#define BuzzBit		3					// 
#define LedOBit		2			// attivo alto; 
#define m_LedOBit		LATAbits.LATA2
#endif


// I2C
#define SPI_232_O		LATBbits.LATB8
#define SPI_232_I		PORTBbits.RB9			//
#define SPI_232_IO		LATBbits.LATB9			//
#define SPI_232_OI		PORTBbits.RB8		// (per I2C clock stretch) 
#define SPI_232_O_TRIS		TRISBbits.TRISB8
#define SPI_232_I_TRIS		TRISBbits.TRISB9
//SPI
#if defined(__PIC24FV32KA302__) || defined(__PIC24EP512GP202__) || defined(__PIC24FJ256GA702__) // ultimi 2 sul conn. est.
#ifdef SPI_USE_CS
#define SPI_232_CS		LATAbits.LATA2
#define SPI_232_CS_TRIS		TRISAbits.TRISA2
#endif
#define SPI_232_O2		LATAbits.LATA3
#define SPI_232_O2_TRIS		TRISAbits.TRISA3
#else			// altri 2 sul conn. i2c/est.
#ifdef SPI_USE_CS
#define SPI_232_CS		LATAbits.LATA1
#define SPI_232_CS_TRIS		TRISAbits.TRISA1
#endif
#define SPI_232_O2		LATBbits.LATB4
#define SPI_232_O2_TRIS		TRISBbits.TRISB4
#endif

#if defined(__PIC24FV32KA302__) || defined(__PIC24EP512GP202__) || defined(__PIC24FJ256GA702__)
#define ZERO_CROSSING_INPUT PORTAbits.RA4		//(PORTA & 0b0000000000010000)		// RA4 (CN0) su schedino 2017; 
#else
#if defined(WIRELESS_MIWI) || defined(WIRELESS_RF24) || defined(WIRELESS_ER900) || defined(WIRELESS_MC13192) || defined(WIRELESS_CC2420)
#define ZERO_CROSSING_INPUT PORTBbits.RB7		//(PORTB & 0b0000000010000000)		// RB7 (CN23) su schedino piccolo 2016 con Wireless; 
#else
#define ZERO_CROSSING_INPUT PORTAbits.RA4		//(PORTA & 0b0000000000010000)		// RA4 (CN0) su schedino piccolo con 485 
#endif
#endif

#define LedOVal		(1 << LedOBit)
#define Led2OVal  (1 << Led2OBit)
#define TriacOVal (1 << TriacOBit)
#define TriacO2Val (1 << TriacO2Bit)
#define TriacO3Val (1 << TriacO3Bit)
#define TriacO4Val (1 << TriacO4Bit)
#define TriacO5Val (1 << TriacO5Bit)
#define TriacO6Val (1 << TriacO6Bit)
#define TXVal			(1 << TXBit)
#define RXVal			(1 << RXBit)
//#define CDVal     (1 << CDBit)
#define Puls1Val  (1 << Puls1Bit)
#define Puls2Val	(1 << Puls2Bit)
#define Puls3Val  (1 << Puls3Bit)
#define Puls4Val  (1 << Puls4Bit)
#define SinusInVal (1 << SinusInBit)
#define AnaInVal  (1 << AnaInBit)
#define BuzzVal		(1 << BuzzBit)

#define TX_RXVal	(1 << TX_RXBit)
#define Led3OVal  (1 << Led3OBit)
#define Led4OVal  (1 << Led4OBit)



#define DELAY_SPI_SLOW 100
//#ifndef SPI_I2C
//#define DELAY_SPI_FAST 1
//#else
#define DELAY_SPI_FAST 5		// qua lasciam cos...
//#endif


#define CR           0x0d
#define LF           0x0a


#ifdef HIGH_SPEED_INPUT				// non  il massimo... MIGLIORARE! (sballa le tempistiche di one_shot e altre notifiche!)
#define FLASH_TIME   1         // 
#else
#define FLASH_TIME   8         // unit da 10mS circa (v.)
#endif

#define TIME_TO_LONG_CLICK 6
#define TIMER_2_TIME 25			// in unit da 10mS * FLASH_TIME
#define AD_NOTIFY_TIME 12		// in unit da 10mS * FLASH_TIME
//#define USA_DORMANT 1


struct COMM_STATUS {
// errori/flag in CommStatus Seriale
	unsigned int FRAME_2SEND:1;         // c'e' da mandare un frame; bit 0
	unsigned int WAIT_4ACK  :1;
	unsigned int FRAME_REC  :1;         // c'e' un frame ricevuto
	unsigned int NO_ACK_REC :1;         // l'ultimo pacch. inviato non ha ricevuto ACK
	unsigned int COMM_OVRERR:1;			// overrun seriale 
	unsigned int COMM_OVL   :1;         // errore di overflow buffer in ricezione 
	unsigned int COMM_TOUT  :1;         // errore di timeout in ricezione frame
	unsigned int COMM_FRERR :1;         // errore di framing (START/STOP) in ricezione 
	unsigned int COMM_PERR  :1;         // errore di parita' in ricezione 
	};

#define VAL_ACCESO 255		// 2017!
#define VAL_SPENTO 0
#define INI_VALUE  VAL_SPENTO
#define HYSTERESIS_AD 12


enum { 
	INVALID_ADDR=0,
	PC_ADDRESS=1,
	TO_ADDR=1,							// per ora sempre il PC... (discorso simile anche per...
	TO_ADDR_SLAVE=1,				// ...notifica master/slave dimmer)
	ADDRESS=2,							// Solo per override/debug: per ora il primo dopo il PC!
	BROADCAST_ADDR=255
	};

#define MY_PANID 0x4700
#define MY_CHANNEL 14

#define SERNUM      1000
#define VERNUMH     4
#define VERNUML     13



// patch varie per PPS...
#if defined(__PIC24EP512GP202__)
// non le prende dagli include, ma porcodiobastardo, sul EP
#define OUT_PIN_PPS_RP36			RPOR1bits.RP36R		
#define OUT_PIN_PPS_RP37			RPOR2bits.RP37R		
#define OUT_PIN_PPS_RP39			RPOR2bits.RP39R		/* Assign RP39 as Output Pin */
#define OUT_PIN_PPS_RP42			RPOR4bits.RP42R		
#define OUT_PIN_PPS_RP43			RPOR4bits.RP43R		
#endif

#if defined(__PIC24FJ256GA702__)
  // puttan'e mammeta, i valori son cambiati!
#define OUT_FN_PPS_OC1				0x000d				/* RPn tied to Output Compare 1 */
#define OUT_FN_PPS_OC2				0x000e				/* RPn tied to Output Compare 2 */
#define OUT_FN_PPS_OC3				0x000f				/* RPn tied to Output Compare 3 */
#endif

#if defined(__PIC24FJ256GA702__) || defined(__PIC24EP512GP202__)
// non le trova... manco in XC... froci urfidi...@$%&
// http://www.microchip.com/forums/m801504.aspx  MBedder !
#define PPSIn(fn,pin)    iPPSInput(IN_FN_PPS##fn,IN_PIN_PPS##pin)
#define PPSOut(fn,pin)    iPPSOutput(OUT_PIN_PPS##pin,OUT_FN_PPS##fn)
#endif

